<?php

namespace Litepci\Okr\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepci\Okr\Http\Requests\ActivityRequest;
use Litepci\Okr\Interfaces\ActivityRepositoryInterface;
use Litepci\Okr\Models\Activity;

/**
 * Resource controller class for activity.
 */
class ActivityResourceController extends BaseController
{

    /**
     * Initialize activity resource controller.
     *
     * @param type ActivityRepositoryInterface $activity
     *
     * @return null
     */
    public function __construct(ActivityRepositoryInterface $activity)
    {
        parent::__construct();
        $this->repository = $activity;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepci\Okr\Repositories\Criteria\ActivityResourceCriteria::class);
    }

    /**
     * Display a list of activity.
     *
     * @return Response
     */
    public function index(ActivityRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepci\Okr\Repositories\Presenter\ActivityPresenter::class)
                ->$function();
        }

        $activities = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('okr::activity.names'))
            ->view('okr::activity.index', true)
            ->data(compact('activities', 'view'))
            ->output();
    }

    /**
     * Display activity.
     *
     * @param Request $request
     * @param Model   $activity
     *
     * @return Response
     */
    public function show(ActivityRequest $request, Activity $activity)
    {

        if ($activity->exists) {
            $view = 'okr::activity.show';
        } else {
            $view = 'okr::activity.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('okr::activity.name'))
            ->data(compact('activity'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new activity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ActivityRequest $request)
    {

        $activity = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('okr::activity.name')) 
            ->view('okr::activity.create', true) 
            ->data(compact('activity'))
            ->output();
    }

    /**
     * Create new activity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ActivityRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $activity                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('okr::activity.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('okr/activity/' . $activity->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/okr/activity'))
                ->redirect();
        }

    }

    /**
     * Show activity for editing.
     *
     * @param Request $request
     * @param Model   $activity
     *
     * @return Response
     */
    public function edit(ActivityRequest $request, Activity $activity)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('okr::activity.name'))
            ->view('okr::activity.edit', true)
            ->data(compact('activity'))
            ->output();
    }

    /**
     * Update the activity.
     *
     * @param Request $request
     * @param Model   $activity
     *
     * @return Response
     */
    public function update(ActivityRequest $request, Activity $activity)
    {
        try {
            $attributes = $request->all();

            $activity->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('okr::activity.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('okr/activity/' . $activity->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('okr/activity/' . $activity->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the activity.
     *
     * @param Model   $activity
     *
     * @return Response
     */
    public function destroy(ActivityRequest $request, Activity $activity)
    {
        try {

            $activity->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('okr::activity.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('okr/activity/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('okr/activity/' . $activity->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple activity.
     *
     * @param Model   $activity
     *
     * @return Response
     */
    public function delete(ActivityRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('okr::activity.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('okr/activity'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/okr/activity'))
                ->redirect();
        }

    }

    /**
     * Restore deleted activities.
     *
     * @param Model   $activity
     *
     * @return Response
     */
    public function restore(ActivityRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('okr::activity.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/okr/activity'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/okr/activity/'))
                ->redirect();
        }

    }

}
